function [xbar,xdot] = ComputeMSDTrim(rho,M,m,k1,k2,b1,b2,Tf)
% function [xbar,xdot] = ComputeMSDTrim(rho,M,m,k1,k2,b1,b2,Tf)
% 
% This function computes an equilibrium state for the nonlinear mass-
% spring-damper system.  The system ODE is described further in the
% function NonlinearMSD.m.  If M>1 the equilibrium state is computed by 
% simulating the system (from zero initial conditions) until it reaches
% a steady state. If M=1 then the equilibrium state can be computed
% directly by solving a cubic equation.
%
% Inputs:
%  *rho = Constant force F applied to block M.  This force parameterizes
%     equilbrium points (xbar(rho),ubar(rho)) with ubar(rho)=0.
%  *M = Number of blocks
%  *m = Mass of individual blocks, kg
%  *k1 = Linear spring constant, N/m
%  *k2 = Cubic spring constant, N/m^3
%  *b1 = Linear damping constant, N/(m/s)
%  *b2 = Cubic damping constant, N/(m/s)^3
%  *Tf = Final time for simulation, sec. 
%          [ Optional input. Default is Tf = 30 ]
%
% Output:
%  *xbar = 2M-by-1 vector specifying the equilibrium point for the
%       given value of rho 
%          xbar = [p1; p2;....; pM; v1; v2; ...; vM]
%       where (pi,vi) are the position (m) and velocity (m/s) of block i.
%  *xdot = 2M-by-1 vector specifying state derivative at xbar, i.e.
%       xdot = f(xbar,ubar,rho).  This should be identically zero if xbar
%       is an exact equilibrium.  The norm  of xdot can be used to verify
%       that xbar is numerically close to being an equilibrium point.
%    

if M==1
    % Single block: Solve cubic equation for trim position. 
    % The cubic equation has only 1 real solution.
    r = roots([k2 0 k1 -rho]);
    [~,idx] = min(abs(imag(r)));
    pbar = r(idx);
    xbar = [pbar; 0];
    xdot = zeros(2,1);
    return
elseif nargin==7
    % Final simulation time, sec
    Tf = 30;           
end

% Input force on block M (in N)
u = [0 0; Tf 0];

% Constant force on block M (in N) specifying equilibrium point
F = [0 rho; Tf rho];

% Function handle to compute the system derivative
xdotfh = @(t,x) NonlinearMSD(t,x,m,k1,k2,b1,b2,F,u);

% Initial Conditions
x0 = zeros(2*M,1);

% Simulate System
opt = odeset;
[t,x]=ode45(xdotfh,[0 Tf],x0,opt);
xbar = x(end,:)';

% Compute state derivative at final time
xdot = xdotfh(0,xbar);
