function [ sysr, V, W, info ] = projred(sys, nr)
%PROJRED [ RSYS, V, W, INFO ] = PROJRED(SYS, ORDERS)
%
%   RSYS = PROJRED(SYS,ORDERS) computes a reduced-order approximation RSYS
%   of the LPV model SYS. The desired order (number of states) for RSYS
%   is specified by ORDERS. You can try multiple orders at once by setting
%   ORDERS to a vector of integers, in which case RSYS is an array of
%   reduced-order models. When SYS is unstable poles, the method cannot be
%   applied.
%
%   V, W are bases for the constant basis space and parameter-varying
%       test space that define the parameter-varying oblique projection to
%       approximate balancing and truncation

% 9/18/2015 Julian Theis


if isa(sys,'pss')
    lpvflag = true;
    sysData = sys.Data(:,:,:);
else
    lpvflag = false;
    sysData = sys(:,:,:);
end

[ny, nu, ngrid] = size(sysData);
 nx             = size(sysData.a,1);
 
%% Calculate Gramians at each Grid Point
method = 'default';
% method = 'krylov';

for kk=1:ngrid
    switch lower(method)
        case 'default' % Hammarling Algorithm
            Lci{kk}  = lyapchol(sysData(:,:,kk).a,sysData(:,:,kk).b)';
            Loi{kk}  = lyapchol(sysData(:,:,kk).a',sysData(:,:,kk).c')';
        case 'krylov'
            Lci{kk}  = IterateArnoldiLyapunovSolver(sysData(:,:,kk).a,sysData(:,:,kk).b*sysData(:,:,kk).b')';
            Loi{kk}  = IterateArnoldiLyapunovSolver(sysData(:,:,kk).a',sysData(:,:,kk).c'*sysData(:,:,kk).c)';
    end
end

%% Calculate Common Dominant Subspace
for jj = 1:size(nr,2)
    LU = zeros(nx,nr(jj)*ngrid);
    SH = zeros(nx,ngrid);
    for kk=1:ngrid
        [Uh, Sh, ~]     = svd(Lci{kk}'*Loi{kk});
         SH(:,kk)       = diag(Sh);
         LU(:,1+nr(jj)*(kk-1):nr(jj)*kk) = Lci{kk}*Uh(:,1:nr(jj));
    end
    [Ubar{jj}, Sbar{jj}, ~]           = svd(LU, 0);
end

if jj>1
    if exist('pmkmp','file') %use more perceptive colormap if available
        col = pmkmp(jj);
    else
        col = colormap;
    end
    figure
    set(gca,'ColorOrder',col)
    for ii=jj:-1:1
        hs = diag(Sbar{ii});
        bar(1:nr(ii),hs(1:nr(ii)),'FaceColor',col(ii,:)); hold on;
    end
    ndesired = input(['Select reduced order [' int2str(nr(1)) '-' int2str(nr(end)) '] : ']);
    jj = find(ndesired==nr);
    Ubar = Ubar{jj};
    Sbar = Sbar{jj};
    nr = ndesired;
else
    Ubar = Ubar{1};
    Sbar = Sbar{1};
end
    

% right projector
V = Ubar(:,1:nr);
% left projector from observability gramians
W = zeros(nx,nr,ngrid);
for kk=1:ngrid
    Y = Loi{kk}'*V;
    W(:,:,kk) = Loi{kk}*Y/(Y'*Y);
end
    

%% Build Reduced-Order System
sysr = ss(zeros(ny,nu,ngrid));
for kk=1:ngrid
    sysr(:,:,kk) = ss(W(:,:,kk)'*sysData(:,:,kk).a*V,...
                          W(:,:,kk)'*sysData(:,:,kk).b,...
                          sysData(:,:,kk).c*V,...
                          sysData(:,:,kk).d);
end
sysr.OutputName = sysData.OutputName;
sysr.InputName = sysData.InputName;


if lpvflag
    sysr = pss(reshape(sysr,[size(sys.Domain) 1]),sys.Domain);
    W    = pmat(reshape(W,[nx nr size(sys.Domain)]),sys.Domain);
end

info.SH   = SH;
info.Sbar = diag(Sbar);
%eof